"""make nullable price column in orders

Revision ID: 5a0ada3a67ed
Revises: e96c68ba9fa5
Create Date: 2021-04-20 16:15:28.097624

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '5a0ada3a67ed'
down_revision = 'e96c68ba9fa5'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('orders', 'price',
               existing_type=sa.NUMERIC(),
               nullable=True)
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.alter_column('orders', 'price',
               existing_type=sa.NUMERIC(),
               nullable=False)
    # ### end Alembic commands ###
